// GraphAlgos 
// Programme implantant les algos de Floyd, Prim, Dijkstra et Bellman-Ford
// graph.cpp : implantation de la classe CGraph
// Date : 08/05/2003
// Auteur: Vouriot Thierry 
// http://thierry.vouriot.free.fr, http://www.yeri.fr.st
// Date : 08/05/2003

#include "graph.h"

// #############
// Classe CGraph
// #############


// Constructeur
CGraph::CGraph()
{
	m_size = 0;
}

// Destructeur
CGraph::~CGraph()	
{ }

// Retourne le nombre de sommets du graphe
int CGraph::GetSize()
{
	return m_size;
}

// Change le nombre de sommets du graphe
void CGraph::SetSize(int size)
{
	m_size = size;
	for (int i=0; i<this->m_size; i++)
		for (int j=0; j<this->m_size; j++)
		{
			m_gr[i][j] = INF;
		}
}

// Retourne l'arc m_gr[i][j]
int CGraph::GetArc(int i, int j)
{
	return m_gr[i][j];
}

// Change l'arc [i][j]  la valeur val
void CGraph::SetArc(int i, int j, int val)
{
	this->m_gr[i][j] = val;
}

// Change l'arete (les 2 arcs dans les deux sens)
void CGraph::SetArete(int i, int j, int val)
{
	this->m_gr[i][j] = val;
	this->m_gr[j][i] = val;
}

// Retourne vrai si un arc i-j existe dans le graphe
bool CGraph::IsArc(int i, int j)	
{
	return (this->m_gr[i][j] != INF);
}

// Teste si i est un sommet du graphe
bool CGraph::IsElem(int i)
{
	return (i < m_size);
}

// Affiche la matrice reprsentant le graphe	
void CGraph::ShowGraph()
{
	int i, j;
	cout << endl << "   ";

	for (i=0; i<m_size; i++) cout << i << "  ";
	cout << endl;
	for (i=0; i<m_size; i++)
	{
		cout << i << "  ";
		for (j=0; j<m_size; j++)
			if (this->m_gr[i][j] != INF)
				cout << m_gr[i][j] << "  ";
			else
				cout << "X  ";
		cout << endl;
	}
}

// Permet de saisir un graphe au "clavier"
void CGraph::SetGraph()
{
	int tmp, size;
	tmp=3;
	cout << "Nombre de sommets du graphe : ";
	cin >> size;
	this->SetSize(size);
	for (int i=0; i<this->m_size; i++)
		for (int j=0; j<this->m_size; j++)
		{
			if(i!=j)
			{
				cout << "Arc entre " << i << " et " << j << " (1 pour oui et 0 pour non) : ";
				cin >> tmp;

				while (tmp < 0 || tmp > 1)
				{
					cout << "1 pour oui et 0 pour non" << endl;
					cin >> tmp;
				}
				if (tmp == 1)
				{
					cout << "Valuation de l'arc entre " << i << " et " << j << " :";
					cin >> m_gr[i][j];
				}
				else 
					this->m_gr[i][j] = INF;
			}
			else this->m_gr[i][j] = INF;
		}
}

// Retourne la valuation minimale de tous les arcs
int CGraph::GetArcMin()
{
	int i,j,min=0;
	for (i=0; i<m_size; i++)
		for (j=0; j<m_size; j++)
			if (this->m_gr[i][j] != INF)
			{	
				min = this->m_gr[i][j];
				break;
			}	

	for (i=0; i<m_size; i++)
		for (j=0; j<m_size; j++)
			if (m_gr[i][j]<=min && i!=j && this->m_gr[i][j] != INF)
				min = m_gr[i][j];

	return min;
}

// Retourne le nombre d'arcs
int CGraph::GetNbArcs()
{
	int x,i,j;
	x=0;
	for (i=0; i<m_size; i++)
		for (j=0; j<m_size; j++)
			if (m_gr[i][j] != INF)
				x++;
	return x;
}

// Retourne une copie du graphe
CGraph * CGraph::Copy()
{
	CGraph * tmp = new CGraph();

	tmp->m_size = this->m_size;
	
	for (int i=0; i<m_size; i++)
		for (int j=0; j<m_size; j++)
		{
			tmp->m_gr[i][j]=this->m_gr[i][j];
		}
	
	return tmp;
}

// Ajoute le sommet size au graphe
void CGraph::AddElem()
{
	m_size++;
	for (int i=0; i<m_size; i++)
	{
		this->m_gr[i][m_size-1] = INF;
		this->m_gr[m_size-1][i] = INF;
	}
}

// Teste si l'ensemble e est egale  l'ensemble des sommets du graphe
bool CGraph::ElemsEqualEns(CEns e)
{
	for (int i=0; i<m_size; i++)
		if (!e.IsElem(i)) return false;
	return true;
}

// Retourne l'ensemble des successeurs d'un sommet 
CEns CGraph::Succ(int x)
{
	CEns ens = CEns();
	for(int j=0; j<m_size; j++)
	{	
		if (IsArc(x,j))
			ens.AddElem(j);
	}
	return ens;
}
