// GraphAlgos
// Programme implantant les algos de Floyd, Prim, Dijkstra et Bellman-Ford
// main.cpp : programme principal contenant les algos
// Date : 08/05/2003
// Auteur: Vouriot Thierry 
// http://thierry.vouriot.free.fr, http://www.yeri.fr.st
// Date : 08/05/2003

#include "main.h"

// Main
int main()
{
	CGraph *graph = new CGraph();
	int choix, choix2, x;

	cout << "\nProjet Maitrise Informatique 2003\nAuteur: Vouriot Thierry\nhttp://thierry.vouriot.free.fr\n\nAlgorithmes s'appliquant sur des graphes simples orientes et values :\n";
	choix = menu();

	while (choix != 7)
	{
		switch (choix)
		{
		case 1:
			cout << "\n";
			graph->SetGraph();
			break;
		case 2:
			cout << "\nGraphe example 1 :\n";
			cout << "non oriente (valuation positive) \navec circuits pour l'algo de prim, bellman-ford, dijkstra ou floyd";
			example1(graph);
			break;
		case 3:
			cout << "\nGraphe example 2 :\n";
			cout << "oriente (valuation positive et negative) \navec circuits pour l'algo de floyd ou bellman-ford";
			example2(graph);
			break;
		case 4:
			cout << "\nGraphe example 3 :\n";
			cout << "oriente (valuation positive et negative) \navec circuit absorbant pour l'algo de bellman-ford";
			example3(graph);
			break;
		case 5:
			cout << "\nGraphe example 4 (vu en TD):\n";
			cout << "non oriente (valuation positive) \npour l'algo de prim, bellman-ford, dijkstra ou floyd";
			example4(graph);
			break;
		case 6:
			cout << "\nGraphe example 5 (vu en TD):\n";
			cout << "oriente (valuation positive) \npour l'algo de bellman-ford, dijkstra ou floyd";
			example5(graph);
			break;
		}

		graph->ShowGraph();
		
		choix2 = menu2();
		switch (choix2)
		{
		case 1:
			floyd(graph);
			break;
		case 2:
			prim(graph);
			break;
		case 3:
			cout << "\nDonner le sommet de depart de l'algo : ? \n"; cin >> x;
			dijkstra(graph,x);
			break;
		case 4:
			cout << "\nDonner le sommet de depart de l'algo : ? \n"; cin >> x;
			bellman_ford(graph,x);
			break;
		}

		cout << "\nAppuyez sur entree pour continuer ...\n";
		getchar();getchar();
		choix = menu();
	}
	
	return 0;
}

// Algorithme de Floyd
// Cet algorithme rsout le problme des plus courtes distances
// de tout sommet du graphe  tout autre sommet, mais ne donne pas
// le chemin correspondant
void floyd(CGraph *graph)
{
	CGraph *res = new CGraph();
	res = graph->Copy();

	for (int i=0; i<res->GetSize(); i++)
		for (int j=0; j<res->GetSize(); j++)
			if (res->IsArc(j, i))
				for (int k=0; k<res->GetSize(); k++)
					if (res->GetArc(j,k) > (res->GetArc(j,i) + res->GetArc(i,k)))
						if (j!=k) res->SetArc(j,k,(res->GetArc(j,i) + res->GetArc(i,k)));

	cout << "\nAlgorithme de Floyd\nMatrice montrant les plus courtes distances\nde tout sommet du graphe a tout autre sommet.";
	res->ShowGraph();

	delete res;
}

// Algorithme de Prim
// Permet de trouver l'arbre couvrant minimal d'un graphe non orient
// La matrice d'adjacences doit donc tre symtrique
void prim(CGraph *graph)
{
	int i,j;
	CGraph *res = new CGraph();	// arbre couvrant minimum
	CEns S = CEns();
	res->SetSize(graph->GetSize());
	
	S.AddElem(0);
	cout << endl << "Algorithme de Prim\n" << "S = "; S.ShowEns();
	
	while (!graph->ElemsEqualEns(S))
	{
		// Cherche l'arete de poids mini partant de res
		int m=-1,n=-1;
		for (i=0; i<graph->GetSize(); i++)
			for (j=0; j<graph->GetSize(); j++)
			{
				if (!S.IsElem(i) && S.IsElem(j))
				{
					if (m==-1) m = i;
					if (n==-1) n = j;
					if (graph->GetArc(i,j) < graph->GetArc(m,n))
					{
						m = i; n = j;
					}

				}

			}
		cout << "Arete de poids minimum dont un seul sommet appartient a S : ("<< m << " " << n << ")" << endl;
		S.AddElem(m);
		cout << "S = "; S.ShowEns();
		res->SetArete(m,n,graph->GetArc(m,n));
	}

	cout << "\nMatrice montrant l'arbre couvrant minimal";
	res->ShowGraph();
}

// Algorithme de Dijkstra
// Permet de trouver les plus courts chemins partant du sommet x  tous
// les autres sommets du graphe (pour un graphe avec valuation positive)
void dijkstra(CGraph *graph, int x)
{
	int y,z,i;
	CEns tmp = CEns();
	CEns C = CEns();
	int *D = new int[graph->GetSize()];
	int *P = new int[graph->GetSize()];

	C.AddElem(x);
	D[x] = 0;
	P[x] = x;

	tmp = graph->Succ(x);
	for (int s=0; s<graph->GetSize(); s++)
		if (s != x)
		{
			if (tmp.IsElem(s)) D[s] = graph->GetArc(x,s);
			else D[s] = INF;
			P[s] = x;
		}

	// -----Affichage------
	cout << "\nAlgorithme de Dijkstra\n";
	cout << "\nLe tableau suivant donne les distances depuis le sommet de depart x\nau cours des differentes etapes de l'algo";
	cout << "\nsommets\t";
	for (i=0; i<graph->GetSize(); i++)
		cout << i << " ";
	cout << "\ninit\t";
	for (i=0; i<graph->GetSize(); i++)
		if (D[i] != INF) cout << D[i] << " ";
		else  cout << "I ";
	// --------------------

	while (C.GetSize() < graph->GetSize())
	{
		y = -1;
		for (i=0; i<graph->GetSize(); i++)
			if (!C.IsElem(i))
			{
				if (y == -1) y = i;
				if (D[i] < D[y]) y = i;
			}
		if (D[y] == INF) break;

		C.AddElem(y);

		for (z=0; z<graph->GetSize(); z++)
			if (!C.IsElem(z))
			{
				if (D[z] > D[y] + graph->GetArc(y,z))
				{
					D[z] = D[y] + graph->GetArc(y,z);
					P[z] = y;
				}
			}
		// -----Affichage------
		cout << "\n\t";
		for (i=0; i<graph->GetSize(); i++)
			if (D[i] != INF) cout << D[i] << " ";
			else  cout << "I ";
		// --------------------
	}
	// -----Affichage------
	cout << endl << endl << "Le tableau suivant contient les peres des sommets\ndans un plus court chemin depuis le sommet x\n";
	for (i=0; i<graph->GetSize(); i++)
		cout << "Predecesseur [" << i <<"] = " << P[i] << "\n";
	// --------------------

	delete [] D;
	delete [] P;
}

#define MIN(a, b)  (((a) < (b)) ? (a) : (b))

// Algorithme de Bellmann-Ford
// Permet de trouver les plus courts chemins partant du sommet x  tous
// les autres sommets du graphe et permet de dtecter des circuits absorbants
void bellman_ford(CGraph *graph, int x)
{
	int u,v,k,i;
	int *D = new int[graph->GetSize()];
	int *P = new int[graph->GetSize()];

	bool circuitabsorbant = false;

	for (int i=0; i<graph->GetSize(); i++)
	{
		D[i] = INF;
		P[i] = x;
	}

	D[x] = 0;
	P[x] = x;

	for (k=1; k<graph->GetSize(); k++)
	{
		for (u=0; u<graph->GetSize(); u++)
			for (v=0; v<graph->GetSize(); v++)
				if (graph->IsArc(u,v))
				{
					if (D[v] > D[u] + graph->GetArc(u,v))
					{
						D[v] = D[u] + graph->GetArc(u,v);
						P[v] = u;
					}
				}
	}

	for (u=0; u<graph->GetSize(); u++)
		for (v=0; v<graph->GetSize(); v++)
			if (graph->IsArc(u,v))
			{
				if (D[v] > D[u] + graph->GetArc(u,v)) 
				{
					circuitabsorbant = true;
					break;
				}
			}

	// -----Affichage------
	cout << "\nAlgorithme de Belman-Ford\n";
	if (!circuitabsorbant) {
		cout << "Le tableau suivant contient les peres des sommets\ndans un plus court chemin depuis le sommet x\n";
		for (i=0; i<graph->GetSize(); i++)
			cout << "Predecesseur [" << i <<"] = " << P[i] << "\n";
	}
	else cout << "Circuit absorbant detecte\n";
	// --------------------

	delete [] D;
	delete [] P;
}


// Menus
int menu()
{
	int choix;

	cout << "\nGraphe :";
	cout << "\n  [1] Saisir manuellement un graphe";
	cout << "\n  [2] Utiliser graphe d'exemple 1 (Prim, Dijkstra, Bellman-Ford ou Floyd)";
	cout << "\n  [3] Utiliser graphe d'exemple 2 (Floyd, Bellman-Ford)";
	cout << "\n  [4] Utiliser graphe d'exemple 3 (Bellman-Ford)";
	cout << "\n  [5] Utiliser graphe d'exemple 4 (Prim, Dijkstra, Bellman-Ford ou Floyd)";
	cout << "\n  [6] Utiliser graphe d'exemple 5 (Dijkstra, Bellman-Ford ou Floyd)";
	cout << "\n  [7] Quitter le programme";
	cout << "\nChoix : ? ";

	cin >> choix;

	while (choix<=0 || choix>7)
	{
		cout << "\nErreur de saisie, entrez votre choix : ? ";
		cin >> choix;
	}

	return choix;
}
int menu2()
{
	int choix;

	cout << "\nAlgorithme a appliquer :";
	cout << "\n  [1] Floyd";
	cout << "\n  [2] Prim";
	cout << "\n  [3] Dijkstra";
	cout << "\n  [4] Bellman-Ford";
	cout << "\n  [5] Revenir au menu principal";
	cout << "\nChoix : ? ";

	cin >> choix;

	while (choix<=0 || choix>5)
	{
		cout << "\nErreur de saisie, entrez votre choix : ? ";
		cin >> choix;
	}

	return choix;
}

// Fonctions creant des graphes utiliss comme exemples

// graphe non orient (valuation positive) avec circuits pour l'algo de prim, dijkstra, bellman-ford et floyd
void example1(CGraph *graph)	
{
	graph->SetSize(5);

	graph->SetArete(0,1,8);
	graph->SetArete(0,2,2);
	graph->SetArete(0,3,1);

	graph->SetArete(1,3,6);
	graph->SetArete(1,2,5);
	graph->SetArete(1,4,1);

	graph->SetArete(2,4,3);
}
// graphe oriente (valuation positive et ngative) avec circuits pour l'algo de floyd, bellman-ford
void example2(CGraph *graph)
{
	graph->SetSize(4);

	graph->SetArc(0,1,2);
	graph->SetArc(1,2,-2);
	graph->SetArc(2,3,5);
	graph->SetArc(0,3,6);
	graph->SetArc(3,0,-4);
	graph->SetArc(3,1,-1);
	graph->SetArc(2,1,5);
}
// graphe oriente (valuation positive et ngative) avec un circuit absorbant pour l'algo de bellman-ford
void example3(CGraph *graph)
{
	graph->SetSize(5);

	graph->SetArc(0,1,3);
	graph->SetArc(0,2,-2);
	graph->SetArc(1,2,1);
	graph->SetArc(2,3,-3);
	graph->SetArc(3,1,1);
	graph->SetArc(2,4,1);
}
// graphe non oriente (valuation positive) pour l'algo de prim, dijkstra, bellman-ford et floyd
void example4(CGraph *graph)
{
	graph->SetSize(9);

	graph->SetArete(0,1,7);
	graph->SetArete(0,2,4);
	graph->SetArete(0,3,1);

	graph->SetArete(1,2,8);
	graph->SetArete(1,4,4);
	graph->SetArete(1,5,2);

	graph->SetArete(2,3,4);
	graph->SetArete(2,4,3);

	graph->SetArete(3,4,3);
	graph->SetArete(3,7,2);

	graph->SetArete(4,5,5);
	graph->SetArete(4,6,1);
	graph->SetArete(4,7,1);

	graph->SetArete(5,8,5);
	graph->SetArete(5,6,8);

	graph->SetArete(6,8,10);
	graph->SetArete(6,7,7);

	graph->SetArete(7,8,3);
}
// graphe oriente (valuation positive) pour l'algo de dijkstra, bellman-ford et floyd
void example5(CGraph *graph)
{
	graph->SetSize(8);

	graph->SetArc(0,1,6);
	graph->SetArc(0,2,4);
	graph->SetArc(0,3,2);

	graph->SetArc(1,4,2);
	graph->SetArc(1,5,6);

	graph->SetArc(2,3,1);
	
	graph->SetArc(3,4,5);
	graph->SetArc(3,6,9);

	graph->SetArc(4,5,5);
	graph->SetArc(4,7,10);
	graph->SetArc(4,2,3);

	graph->SetArc(5,7,5);

	graph->SetArc(6,4,4);
	graph->SetArc(6,7,5);
}