// GraphAlgos
// Programme implantant les algos de Floyd, Prim, Dijkstra et Bellman-Ford
// main.h
// Date : 08/05/2003
// Auteur: Vouriot Thierry 
// http://thierry.vouriot.free.fr, http://www.yeri.fr.st
// Date : 08/05/2003

#ifndef _MAIN_H
#define _MAIN_H

// Inclusions standards
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <math.h>
#include <iostream>

using namespace std;

// Windows inclusions
#ifdef _WIN32
#include <windows.h>
#define WIN32_LEAN_AND_MEAN
#endif

// Inclusions persos
#include "graph.h"
#include "ens.h"


// Prototypes

// Algorithme de Floyd
// Cet algorithme rsout le problme des plus courtes distances
// de tout sommet du graphe  tout autre sommet, mais ne donne pas
// le chemin correspondant
void floyd(CGraph *graph);

// Algorithme de Prim
// Permet de trouver l'arbre couvrant minimal d'un graphe non orient
void prim(CGraph *graph);

// Algorithme de Dijkstra
// Permet de trouver les plus courts chemins partant du sommet x  tous
// les autres sommets du graphe (pour un graphe avec valuation positive)
void dijkstra(CGraph *graph, int x);

// Algorithme de Bellmann-Ford
// Permet de trouver les plus courts chemins partant du sommet x  tous
// les autres sommets du graphe et permet de dtecter des circuits absorbants
void bellman_ford(CGraph *graph, int x);

// Menus
int menu();
int menu2();

// Fonctions creant des graphes utiliss comme exemples
// graphe non orient (valuation positive) avec circuits pour l'algo de prim, dijkstra et floyd
void example1(CGraph *graph);
// graphe oriente (valuation positive et ngative) avec circuits pour l'algo de floyd
void example2(CGraph *graph);
// graphe oriente (valuation positive et ngative) avec un circuit absorbant pour l'algo de bellman-ford
void example3(CGraph *graph);
// graphe non oriente (valuation positive) pour l'algo de prim, dijkstra, bellman-ford et floyd
void example4(CGraph *graph);
// graphe oriente (valuation positive) pour l'algo de dijkstra, bellman-ford et floyd
void example5(CGraph *graph);

#endif
